package pro.shushi.pamirs.demo.core.session;

import pro.shushi.pamirs.business.api.model.PamirsEmployee;
import pro.shushi.pamirs.demo.api.model.PetCompany;
import pro.shushi.pamirs.demo.api.service.PetCompanyQueryService;
import pro.shushi.pamirs.demo.api.service.PetEmployeeQueryService;
import pro.shushi.pamirs.meta.api.CommonApiFactory;
import pro.shushi.pamirs.meta.api.session.PamirsSession;
import pro.shushi.pamirs.user.api.login.UserInfoCache;
import pro.shushi.pamirs.user.api.model.PamirsUser;

/**
 * *********************************************************************************
 * *
 * 这是个类似于AOP的类，这里慢，系统就会变慢，请注意修改！！                              *
 * 数据尽快能的从缓存中获取，同时注意数据变更后及时同步清理缓存以便下次获取到最新的数据！！     *
 * 示例代码仅仅是示例，未从缓存中取，未完全考虑数据为空的情况
 * *
 * *********************************************************************************
 */
public class DemoSessionCache {
    private static final ThreadLocal<DemoSessionData> BIZ_DATA_THREAD_LOCAL = new ThreadLocal<>();

    public static PamirsUser getUser() {
        return BIZ_DATA_THREAD_LOCAL.get() == null ? null : BIZ_DATA_THREAD_LOCAL.get().getUser();
    }

    public static PetCompany getCompany() {
        return BIZ_DATA_THREAD_LOCAL.get() == null ? null : BIZ_DATA_THREAD_LOCAL.get().getCompany();
    }

    public static void init() {
        if (getUser() != null) {
            return;
        }
        Long uid = PamirsSession.getUserId();
        if (uid == null) {
            // 根据实际情况抛异常获取其他逻辑
            return;
        }
        PamirsUser user = UserInfoCache.queryUserById(uid);
        if (user != null) {
            DemoSessionData demoSessionData = new DemoSessionData();
            demoSessionData.setUser(user);

            PamirsEmployee employee = CommonApiFactory.getApi(PetEmployeeQueryService.class).queryByBingDingUserId(uid);
            if (employee != null) {
                PetCompany company = CommonApiFactory.getApi(PetCompanyQueryService.class).queryByCode(employee.getCompanyCode());
                if (company!=null) {
                    demoSessionData.setCompany(company);
                } else {
                    demoSessionData.setCompany(mockPetCompany());
                }
            } else {
                demoSessionData.setCompany(mockPetCompany());
            }
            BIZ_DATA_THREAD_LOCAL.set(demoSessionData);
        }
    }

    public static void clear(){
        BIZ_DATA_THREAD_LOCAL.remove();
    }

    private static PetCompany mockPetCompany() {
        PetCompany petCompany = new PetCompany();
        petCompany.setCode("C001");
        petCompany.setName("name001");
        return petCompany;
    }
}